import struct
import json
import sys
import os
try:
	# try with a fast c-implementation ...
	import mmh3
except ImportError:
	# ... otherwise fallback to this code!
	try:
		import pymmh3 as mmh3
	except:
		print("Not detected any of supported mmh3 libraries.")
		print("Try to install necessary library by using command:")
		print("pip install mmh3")
		print("or")
		print("pip install pymmh3")
		input()
		sys.exit()

file = open("lib_nx.json", "r", encoding="UTF-8")
try:
	DUMP = json.load(file)
except Exception as exc:
	print("Detected malformed lib_nx.json")
	print(exc)
	file.close()
	input("Press any key to continue")
	sys.exit()
file.close()

keys = list(DUMP.keys())

header = [b"INIB", int.to_bytes(10000, 4, byteorder="little"), int.to_bytes(0x10, 4, byteorder="little"), int.to_bytes(len(keys), 4, byteorder="little")]

entry_table = []
binary_list = []

binary_start = 0x10 + (0xC * len(keys))

for i in range(0, len(keys)):
	if (keys[i][0:2] != "0x"):
		entry_table.append(int.to_bytes(mmh3.hash(keys[i]), 4, byteorder="little", signed=True))
	else:
		entry_table.append(int.to_bytes(int(keys[i], base=16), 4, byteorder="little"))
	if (type(DUMP[keys[i]]) is bool):
			entry_table.append(int.to_bytes(0, 4, byteorder="little"))
			entry_table.append(int.to_bytes(binary_start, 4, byteorder="little"))
			binary_start += 4
			binary_list.append(int.to_bytes(DUMP[keys[i]], 4, byteorder="little"))
	elif (type(DUMP[keys[i]]) is float):
			entry_table.append(int.to_bytes(1, 4, byteorder="little"))
			entry_table.append(int.to_bytes(binary_start, 4, byteorder="little"))
			binary_start += 4
			binary_list.append(struct.pack('f', DUMP[keys[i]]))
	elif (type(DUMP[keys[i]]) is list):
		match(len(DUMP[keys[i]])):
			case 3:
				entry_table.append(int.to_bytes(2, 4, byteorder="little"))
				entry_table.append(int.to_bytes(binary_start, 4, byteorder="little"))
				binary_start += 12
			case 4:
				entry_table.append(int.to_bytes(3, 4, byteorder="little"))
				entry_table.append(int.to_bytes(binary_start, 4, byteorder="little"))
				binary_start += 16
			case _:
				print("Wrong data count for key %s. Malformed JSON file?" % keys[i])
				input("Press any key to continue")
				sys.exit()
		
		for x in range(0, len(DUMP[keys[i]])):
			binary_list.append(struct.pack('f', DUMP[keys[i]][x]))

	else:
		print("Unknown type for key %s. Malformed JSON file?" % keys[i])
		input("Press any key to continue")
		sys.exit()

os.makedirs("atmosphere/contents/010074f013262000/romfs/stream/dumpbin", exist_ok=True)
new_file = open("atmosphere/contents/010074f013262000/romfs/stream/dumpbin/lib_nx.bin", "wb")
new_file.write(b"".join(header))
new_file.write(b"".join(entry_table))
new_file.write(b"".join(binary_list))
new_file.close()